/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.basis.VerilogExpression;

import java.util.EnumSet;

/**
 * {@link VerilogAttribute} represents attributes.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogAttribute extends VerilogNode {
  public static final Tag TAG = Tag.ATTRIBUTE;
  public static final EnumSet<Tag> TAGS_CHILDREN = EnumSet.noneOf(Tag.class);

  /** Expression of the attribute. */
  private VerilogExpression expression;

  /**
   * Creates an attribute.
   * 
   * @param parent the node parent.
   */
  public VerilogAttribute(final VerilogNode parent) {
    super(TAG, TAGS_CHILDREN, NodeKind.ELEMENT, parent);
  }

  /**
   * Creates a copy of the attribute.
   * 
   * @param other the attribute to be copied.
   * @param parent the parent node.
   */
  public VerilogAttribute(final VerilogAttribute other, final VerilogNode parent) {
    super(other, parent);

    this.expression = other.expression != null ? other.expression.clone() : null;
  }

  /**
   * Returns the expression of the attribute.
   * 
   * @return the expression.
   */
  public VerilogExpression getExpression() {
    return expression;
  }

  /**
   * Sets the expression of the attribute.
   * 
   * @param expression the expression to be set.
   */
  public void setExpression(final VerilogExpression expression) {
    InvariantChecks.checkNotNull(expression);
    this.expression = expression;
  }

  @Override
  public VerilogAttribute clone() {
    return new VerilogAttribute(this, getParentNode());
  }
}
