/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import java.util.EnumSet;

/**
 * {@link VerilogNullStatement} represents null statements.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogNullStatement extends VerilogStatement {
  public static final Tag TAG = Tag.NULL_STATEMENT;
  public static final EnumSet<Tag> TAGS_CHILDREN = EnumSet.noneOf(Tag.class);

  /**
   * Creates a null statement.
   * 
   * @param parent the parent node.
   */
  public VerilogNullStatement(final VerilogNode parent) {
    super(TAG, parent);
  }

  /**
   * Creates a copy of the null statement.
   * 
   * @param other the null statement to be copied.
   * @param parent the parent node.
   */
  public VerilogNullStatement(final VerilogNullStatement other, final VerilogNode parent) {
    super(other, parent);
  }

  @Override
  public VerilogNullStatement clone() {
    return new VerilogNullStatement(this, getParentNode());
  }
}
