/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser;

import org.junit.Test;

import java.io.File;
import java.nio.file.Paths;

/**
 * Verilog2Smv\VIS benchmark jUnit test cases.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class VerilogVisVerilog2SmvTestCase extends VerilogBenchmarkTest {

  /* Am2901 */

  @Test
  public void runTest_Am2901_am2901() {
    runTest("am2901", Paths.get("Am2901", "am2901.v").toString());
  }

  /* Am2910 */

  @Test
  public void runTest_Am2910_am2910() {
    runTest("am2910", Paths.get("Am2910", "am2910.v").toString());
  }

  /* Bpb */

  @Test
  public void runTest_Bpb_bpbs() {
    runTest("branchPredictionBuffer", Paths.get("Bpb", "bpbs.v").toString());
  }

  /* BufAl */

  @Test
  public void runTest_BufAl_buf_bug() {
    runTest("buffer_alloc", Paths.get("BufAl", "buf_bug.v").toString());
  }

  @Test
  public void runTest_BufAl_bufferAlloc() {
    runTest("buffer_alloc", Paths.get("BufAl", "bufferAlloc.v").toString());
  }

  /* Field */

  @Test
  public void runTest_Field_field5() {
    runTest("field5", Paths.get("Field", "field5.v").toString());
  }

  /* FIFOs */

  @Test
  public void runTest_Fifos_Fifos() {
    runTest("compareFIFOs", Paths.get("FIFOs", "FIFOs.v").toString());
  }

  /* FourByFour */

  @Test
  public void runTest_FourByFour_two() {
    runTest("twoByFour", Paths.get("FourByFour", "two.v").toString());
  }

  /* Ibuf */

  @Test
  public void runTest_Ibuf_ibuf() {
    runTest("iqc", Paths.get("Ibuf", "ibuf.v").toString());
  }

  /* Miim */

  @Test
  public void runTest_Miim_vMiim() {
    runTest("miim", Paths.get("Miim", "vMiim.v").toString());
  }

  /* Palu */

  @Test
  public void runTest_Palu_palu() {
    runTest("palu", Paths.get("Palu", "palu.v").toString());
  }

  /* Rotate */

  @Test
  public void runTest_Rotate_rotate32() {
    runTest("rotate", Paths.get("Rotate", "rotate32.v").toString());
  }

  /* s1269 */

  @Test
  public void runTest_s1269_s1269b() {
    runTest("s1269", Paths.get("s1269", "s1269b.v").toString());
  }

  /* Sampleq */

  @Test
  public void runTest_Sampleq_twoFifo1() {
    runTest("sampleq", Paths.get("Sampleq", "twoFifo1.v").toString());
  }

  /* Silver-bcu */

  @Test
  public void runTest_Silver_bcu_bcuvis32() {
    runTest("bcu", Paths.get("Silver-bcu", "bcuvis32.v").toString());
  }

  /* Silver-fru */

  @Test
  public void runTest_Silver_fru_includes_const() {
    runTest(null, Paths.get("Silver-fru", "includes", "const.v").toString());
  }

  @Test
  public void runTest_Silver_fru_includes_decode() {
    runTest(null, Paths.get("Silver-fru", "includes", "decode.v").toString());
  }

  @Test
  public void runTest_Silver_fru_includes_opcode() {
    runTest(null, Paths.get("Silver-fru", "includes", "opcode.v").toString());
  }

  @Test
  public void runTest_Silver_fru_fru32() {
    runTest("fru",
        Paths.get("Silver-fru", "includes", "const.v").toString(),
        Paths.get("Silver-fru", "includes", "decode.v").toString(),
        Paths.get("Silver-fru", "includes", "opcode.v").toString(),
        Paths.get("Silver-fru", "fru32.v").toString());
  }

  /* Spinner */

  @Test
  public void runTest_Spinner_spinner32() {
    runTest("spinner", Paths.get("Spinner", "spinner32.v").toString());
  }

  /* Vlunc */

  @Test
  public void runTest_Vlunc_vlunc() {
    runTest("lunc", Paths.get("Vlunc", "vlunc.v").toString());
  }

  /* Vsa16 */

  @Test
  public void runTest_Vsa16_vsa16a() {
    runTest("vsa16a", Paths.get("Vsa16", "vsa16a.v").toString());
  }

  /* VsaR */

  @Test
  public void runTest_VsaR_vsaR() {
    runTest("vsaR", Paths.get("VsaR", "vsaR.v").toString());
  }

  @Override
  String getTestDirPath() {
    return System.getProperty("user.dir")
        + File.separator
        + Paths.get("src", "test", "verilog", "verilog2smv-vis-tests").toString()
        + File.separator;
  }
}
