/*
 * Copyright 2017-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.verilog.parser.sample;

import org.junit.Test;

import java.io.File;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

/**
 * Basic class for jUnit test cases that run {@link VerilogPrinter}.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class VerilogPrinterTestCase {

  @Test
  public void runTest() {

    final String[] paths = getPaths();
    final String[] fullPaths = new String[paths.length];

    for (int i = 0; i < paths.length; i++) {
      fullPaths[i] = getFullPathPrefix() + paths[i];
    }

    final List<String> commandLine = new LinkedList<>();
    commandLine.addAll(Arrays.asList(getOptions()));
    commandLine.addAll(Arrays.asList(fullPaths));

    VerilogPrinter.main(commandLine.toArray(new String[commandLine.size()]));
  }

  public abstract String[] getOptions();

  public abstract String[] getPaths();

  public static String getFullPathPrefix() {
    return System.getProperty("user.dir")
        + File.separator
        + Paths.get("src", "test", "verilog").toString()
        + File.separator;
  }
}
