module altsyncram
  #(
    //----------------------------------------------------------------------
    parameter operation_mode = "DUAL_PORT",
    parameter lpm_type = "altsyncram",
    parameter read_during_write_mode_mixed_ports = "DONT_CARE",
    parameter power_up_uninitialized = "TRUE",
    parameter byte_size = 8,
    parameter width_byteena_a = 4,
    parameter numwords_a = 128,
    parameter numwords_b = 128,
    parameter width_a = 32,     // Data Width in bits
    parameter width_b = 32,     // Data Width in bits
    parameter widthad_a = 7,     // Address Width in bits
    parameter widthad_b = 7,     // Address Width in bits
    parameter address_reg_b = "CLOCK0",
    parameter outdata_reg_b = "UNREGISTERED"

    //----------------------------------------------------------------------
    ) (
    input clock0,
    input wren_a,
    input [width_byteena_a-1:0] byteena_a,
    input [width_a-1:0] data_a,
    input [widthad_a-1:0] address_a,
    output [width_b-1:0] q_b,
    input [widthad_b-1:0] address_b
       );

   sdp_ram #(
      .COL_WIDTH (byte_size),
      .DATA_WIDTH (width_a),
      .ADDR_WIDTH (widthad_a),
      .NUM_COL (width_byteena_a),
      .OUTDATA_REG (outdata_reg_b)
   ) ram_model (
      .clkA(clock0),
      .enaA(wren_a),
      .weA(byteena_a),
      .addrA(address_a),
      .dinA(data_a),
      .doutA(),
      .clkB(clock0),
      .enaB(1'b1),
      .weB({width_byteena_a{1'b0}}),
      .addrB(address_b),
      .dinB({width_b{1'b0}}),
      .doutB(q_b)
   );

endmodule

module sdp_ram
   #(
   parameter COL_WIDTH = 8,
   parameter DATA_WIDTH = 32,
   parameter ADDR_WIDTH = 7,
   parameter NUM_COL = 4,
   parameter OUTDATA_REG = "UNREGISTERED"
   ) (
       input 			   clkA,
       input 			   enaA,
       input [NUM_COL-1:0] 	   weA,
       input [ADDR_WIDTH-1:0] 	   addrA,
       input [DATA_WIDTH-1:0] 	   dinA,
       output [DATA_WIDTH-1:0]     doutA,
       input 			   clkB,
       input 			   enaB,
       input [NUM_COL-1:0] 	   weB,
       input [ADDR_WIDTH-1:0] 	   addrB,
       input [DATA_WIDTH-1:0] 	   dinB,
       output [DATA_WIDTH-1:0]     doutB
   );

   // Core Memory
   reg [DATA_WIDTH-1:0] 	   sdpram_ben_outreg [(2**ADDR_WIDTH)-1:0];
   reg [DATA_WIDTH-1:0] 	   sdpram_ben [(2**ADDR_WIDTH)-1:0];   
   reg [DATA_WIDTH-1:0] 	   ram_data;
   reg [DATA_WIDTH-1:0] 	   doutB_r;
   reg [DATA_WIDTH-1:0] 	   doutA_r;
   integer 			   i;
   // Port-A Operation
   generate
      if (OUTDATA_REG != "UNREGISTERED")
	begin
	   assign doutA = doutA_r;
	   assign doutB = doutB_r;
	   always @ (posedge clkA) begin
	      if(enaA) begin
		 for(i=0;i<NUM_COL;i=i+1) begin
		    if(weA[i]) begin
		       sdpram_ben_outreg[addrA][i*COL_WIDTH +: COL_WIDTH] <= dinA[i*COL_WIDTH +: COL_WIDTH];
		    end
		 end
		 doutA_r <= sdpram_ben_outreg[addrA];
	      end
	   end
	   // Port-B Operation:
	   always @ (posedge clkB) begin
	      if(enaB) begin
		 for(i=0;i<NUM_COL;i=i+1) begin
		    if(weB[i]) begin
		    end
		 end
		 ram_data <= sdpram_ben_outreg[addrB];
		 doutB_r <= ram_data;
	      end
	   end                     
       end
	       
      else
	begin
	   assign doutA = doutA_r;
	   assign doutB = doutB_r;
	   always @ (posedge clkA) begin
	      if(enaA) begin
		 for(i=0;i<NUM_COL;i=i+1) begin
		    if(weA[i]) begin
		       sdpram_ben[addrA][i*COL_WIDTH +: COL_WIDTH] <= dinA[i*COL_WIDTH +: COL_WIDTH];
		    end
		 end
		 doutA_r <= sdpram_ben[addrA];
	      end
	   end
	   // Port-B Operation:
	   always @ (posedge clkB) begin
	      if(enaB) begin
		 doutB_r <= sdpram_ben[addrB];
	      end
	    end 
    end
   endgenerate


endmodule 
